/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIThread.idl"
#include "nsIPrincipal.idl"

%{C++
#include "nsTArray.h"
#include "nsStringGlue.h"
class GMPVideoDecoder;
class GMPVideoEncoder;
class GMPVideoHost;
%}

[ptr] native GMPVideoDecoder(GMPVideoDecoder);
[ptr] native GMPVideoEncoder(GMPVideoEncoder);
[ptr] native GMPVideoHost(GMPVideoHost);
[ptr] native MessageLoop(MessageLoop);
[ptr] native TagArray(nsTArray<nsCString>);

[uuid(BF5A9086-70F5-4D38-832D-1609BBF963CD)]
interface mozIGeckoMediaPluginService : nsISupports
{
  // Returns the GMP thread.
  // Callable from any thread.
  readonly attribute nsIThread thread;

  // Returns a video decoder that supports the specified tags.
  // The array of tags should at least contain a codec tag, and optionally
  // other tags such as for EME keysystem.
  // Callable only on GMP thread.
  GMPVideoDecoder getGMPVideoDecoder(in TagArray tags,
                                     [optional] in AString origin,
                                     out GMPVideoHost outVideoHost);

  // Returns a video encoder that supports the specified tags.
  // The array of tags should at least contain a codec tag, and optionally
  // other tags.
  // Callable only on GMP thread.
  GMPVideoEncoder getGMPVideoEncoder(in TagArray tags,
                                     [optional] in AString origin,
                                     out GMPVideoHost outVideoHost);
};
