// Temporary shim to keep a couple dependencies working in Chromium.
#ifndef SkFlattenableBuffers_DEFINED
#define SkFlattenableBuffers_DEFINED

#include "SkReadBuffer.h"
#include "SkWriteBuffer.h"

typedef SkReadBuffer SkFlattenableReadBuffer;

#endif//SkFlattenableBuffers_DEFINED
