/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(15d424c0-6b47-11d9-9791-000a95dc234c)]
interface txINodeSet : nsISupports
{
    nsIDOMNode item(in unsigned long index);
    double itemAsNumber(in unsigned long index);
    DOMString itemAsString(in unsigned long index);
    readonly attribute unsigned long length;
    void add(in nsIDOMNode node);
};
